==> scanbot_sdk-2.6.0-beta4.tar.gz: lib/scanbot_sdk.dart <==
import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:logging/logging.dart';
import 'package:scanbot_sdk/create_tiff_data.dart';
import 'package:scanbot_sdk/license.dart';
import 'package:scanbot_sdk/ocr_data.dart';
import 'package:scanbot_sdk/render_pdf_data.dart';
import 'package:scanbot_sdk/scanbot_sdk_models.dart';
import 'package:scanbot_sdk/scanbot_sdk_ui.dart';
import 'package:scanbot_sdk/sdk_utils.dart';

import 'barcode_scanning_data.dart';
import 'common_data.dart';

class ScanbotSdk {
  static const MethodChannel _channel = MethodChannel('scanbot_sdk');

  static Function(Status, SdkFeature?, String)? _sdkLicenseListener;

  /// Initializes the Scanbot SDK with given [config]s.
  /// The Scanbot SDK must be initialized before using any other API methods.
  static Future<void> initScanbotSdk(ScanbotSdkConfig config) async {
    LicenseRegistry.addLicense(licenses);

    try {
      _sdkLicenseListener = config.licenseErrorHandler;
      _channel.setMethodCallHandler(_handler);
      var json = config.toJson();
      await _channel.invokeMethod('initScanbotSdk', json);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  static final Handler _handler = (MethodCall call) async {
    switch (call.method) {
      case 'onLicenseError':
        var e = SdkLicenseException.fromJson(jsonDecode(call.arguments));
        _sdkLicenseListener?.call(e.status, e.feature, e.message);
        break;
    }
  };

  /// Performs auto document detection on the ORIGINAL image of the given [page]
  /// and creates (cropped) DOCUMENT image.
  /// The given [page] must have the ORIGINAL image.
  /// return [Page] with detected polygon.
  static Future<Page> detectDocument(Page page) async {
    try {
      var arguments = {'page': page.toJson()};
      var documentScanningResult =
          await _channel.invokeMethod('detectDocument', arguments);
      var json = jsonDecode(documentScanningResult);
      var pageResult = Page.fromJson(json);
      return pageResult;
    } catch (e) {
      return page;
    }
  }

  /// Returns the current license status of the Scanbot SDK.
  static Future<SdkLicenseStatus> getLicenseStatus() async {
    try {
      var licenseStatus = await _channel.invokeMethod('getLicenseStatus', {});
      var json = jsonDecode(licenseStatus);
      return SdkLicenseStatus.fromJson(json);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Renders a multi-page PDF document from DOCUMENT images of the given [pages]
  /// and stores it as a file. For each DOCUMENT image a separate PDF page is generated.
  /// All given [pages] must have the DOCUMENT images.
  ///
  /// [pages] list of pages to render
  /// [options] additional options of rendering
  /// return File [Uri] of the PDF document.
  static Future<Uri> createPdf(
      List<Page> pages, PdfRenderingOptions options) async {
    try {
      var pagesList = pages.map((page) => page.toJson()).toList();
      var arguments = {'pages': pagesList, 'options': options.toJson()};
      final uri = await _channel.invokeMethod('createPdf', arguments);
      return Uri.parse(uri);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Renders a multi-page TIFF image from DOCUMENT images of the given [pages]
  /// and stores it as a file. For each DOCUMENT image a separate TIFF page is generated.
  /// All given [pages] must have the DOCUMENT images.
  ///
  /// [pages] list of pages to render
  /// [options] additional options of rendering
  static Future<Uri> createTiff(
      List<Page> pages, TiffCreationOptions options) async {
    try {
      if (!options.binarized &&
          PredefinedCompressionTypes.binarizedOnlyCompressionTypes()
              .contains(options.compression)) {
        throw ('Incompatible compression type. The given compression type ${options.compression} is only applicable to binarized images.');
      }

      var pagesList = pages.map((page) => page.toJson()).toList();
      var arguments = {'pages': pagesList, 'options': options.toJson()};
      final uri = await _channel.invokeMethod('createTiff', arguments);
      return Uri.parse(uri);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Cleanups the storage folders of the Scanbot SDK.
  /// By calling this function all output files generated by the native Scanbot SDKs
  /// as well as by this Plugin will be deleted, including the image files of all scanned
  /// or imported Page objects, all generated PDF files, TIFF files, etc.
  static Future<void> cleanupStorage() async {
    try {
      await _channel.invokeMethod('cleanupStorage');
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Removes a page with all its files (ORIGINAL image, DOCUMENT image,
  /// previews/thumbnails, filtered images, etc) from the file storage of the Scanbot SDK.
  /// This method does not remove any exported files (PDF, TIFF, etc) which were
  /// generated based on this page object.
  static Future<void> deletePage(Page page) async {
    try {
      var arguments = {'page': page.toJson()};
      await _channel.invokeMethod('deletePage', arguments);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Creates a page from an image file URI. The given image will be used as ORIGINAL image.
  /// Use the [shouldDetectDocument] flag to specify whether the auto-document detection
  /// should be performed and the (cropped) DOCUMENT image should be created.
  /// return [Page]
  static Future<Page> createPage(
      Uri originalImageFileUri, bool shouldDetectDocument) async {
    try {
      var arguments = {
        'originalImageFileUri': originalImageFileUri.toString(),
        'shouldDetectDocument': shouldDetectDocument
      };
      final pageJson = await _channel.invokeMethod('createPage', arguments);
      var decoded = jsonDecode(pageJson);
      return Page.fromJson(decoded);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Rotates all images of the given [page] by 90 degrees value for [rotationsCount] times.
  static Future<Page> rotatePageClockwise(Page page, int rotationsCount) async {
    try {
      var arguments = {'page': page.toJson(), 'rotationsCount': rotationsCount};
      var pageJson =
          await _channel.invokeMethod('rotatePageClockwise', arguments);
      var decoded = jsonDecode(pageJson);
      var rotatedPage = Page.fromJson(decoded);
      return rotatedPage;
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Applies an image [filter] on the final DOCUMENT image of given [page].
  static Future<Page> applyImageFilter(
      Page page, ImageFilterType filter) async {
    try {
      var arguments = {
        'page': page.toJson(),
        'filter': imageFilterJsonValue(filter)
      };
      final pageJson =
          await _channel.invokeMethod('applyImageFilter', arguments);
      var decoded = jsonDecode(pageJson);
      return Page.fromJson(decoded);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Creates a preview image file of the DOCUMENT image by given [filter].
  /// Does not modify the actual DOCUMENT image of the passed [page].
  /// Use the 'applyImageFilter(page, filter)' method to finally apply an image filter on a page.
  static Future<Uri> getFilteredDocumentPreviewUri(
      Page page, ImageFilterType filter) async {
    try {
      var arguments = {
        'page': page.toJson(),
        'filter': imageFilterJsonValue(filter)
      };
      final uri = await _channel.invokeMethod(
          'getFilteredDocumentPreviewUri', arguments);
      return Uri.parse(uri);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Returns current OCR configs, like installed OCR languages, etc.
  static Future<SdkOcrConfig> getOcrConfigs() async {
    try {
      var result = await _channel.invokeMethod('getOcrConfig');
      var json = jsonDecode(result);
      return SdkOcrConfig.fromJson(json);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Performs OCR (Optical Character Recognition) on DOCUMENT images of given pages.
  static Future<OcrResult> performOcr(
      List<Page> pages, OcrOptions options) async {
    try {
      var arguments = {
        'pages': pages.map((e) => e.toJson()).toList(),
        'options': options.toJson()
      };

      var result = await _channel.invokeMethod('performOcr', arguments);
      var json = jsonDecode(result);
      return OcrResult.fromJson(json);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Refreshes image file URIs of given pages. For more details please
  /// check out the section 'Persistence of Page Objects' in the documentation.
  static Future<List<Page>> refreshImageUris(List<Page> pages) async {
    try {
      final jsonPages = pages.map((page) => page.toJson()).toList();
      final result =
          await _channel.invokeMethod('refreshImageUris', {'pages': jsonPages});
      final json = jsonDecode(result);
      return (json['pages'] as List?)
              ?.map((p) => Page.fromJson(p as Map<String, dynamic>))
              .toList() ??
          <Page>[];
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Detect barcodes  from image file uri
  /// The Barcode Scanner is able to recognize 1D- and 2D-barcodes (QR codes, etc).
  ///image [rotation]  between 0-360 degrees with 90 degree step
  /// [barcodeFormats] Format of barcode which  need to recognize
  static Future<BarcodeScanningResult> detectBarcodeFromImageFile(
      Uri file,
      List<BarcodeFormat> barcodeFormats) async {
    try {
      var detectFromImageParams = DetectBarcodeFromImageParams(
          file, barcodeFormats);
      var barcodeScanningResult = await _channel.invokeMethod(
          'detectBarcodesFromImageFile', detectFromImageParams.toJson());
      return BarcodeScanningResult.fromJson(jsonDecode(barcodeScanningResult));
    } catch (e) {
      Logger.root.severe(e);
      return BarcodeScanningResult.error(e.toString());
    }
  }

  /// Estimate blurriness on a given page.
  /// The document image will be analyzed, if possible. If there is no document image - then the original image will be used.
  /// Returns the value between 0.0 and 1.0. Values closer to 0.0 mean that the image is sharper, values closer to 1.0 - the image is blurred.
  static Future<double> estimateBlurOnPage(Page page) async {
    try {
      var arguments = {'page': page.toJson()};
      final value =
          await _channel.invokeMethod('estimateBlurOnPage', arguments);
      return value;
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }
}

==> scanbot_sdk-2.10.0-beta4.tar.gz: lib/scanbot_sdk.dart <==
import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:logging/logging.dart';
import 'package:scanbot_sdk/barcode_scanning_data.dart';
import 'package:scanbot_sdk/create_tiff_data.dart';
import 'package:scanbot_sdk/license.dart';
import 'package:scanbot_sdk/ocr_data.dart';
import 'package:scanbot_sdk/render_pdf_data.dart';
import 'package:scanbot_sdk/scanbot_sdk_models.dart';
import 'package:scanbot_sdk/scanbot_sdk_ui.dart';
import 'package:scanbot_sdk/sdk_utils.dart';

import 'json/common_data.dart';

class ScanbotSdk {
  static const MethodChannel _channel = MethodChannel('scanbot_sdk');

  static Function(Status, SdkFeature?, String)? _sdkLicenseListener;

  /// Initializes the Scanbot SDK with given [config]s.
  /// The Scanbot SDK must be initialized before using any other API methods.
  static Future<void> initScanbotSdk(ScanbotSdkConfig config) async {
    LicenseRegistry.addLicense(licenses);

    try {
      _sdkLicenseListener = config.licenseErrorHandler;
      _channel.setMethodCallHandler(_handler);
      var json = config.toJson();
      await _channel.invokeMethod('initScanbotSdk', json);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  static Handler get _handler => (MethodCall call) async {
        switch (call.method) {
          case 'onLicenseError':
            var e = SdkLicenseException.fromJson(jsonDecode(call.arguments));
            _sdkLicenseListener?.call(e.status, e.feature, e.message);
            break;
        }
      };

  /// Performs auto document detection on the ORIGINAL image of the given [page]
  /// and creates (cropped) DOCUMENT image.
  /// The given [page] must have the ORIGINAL image.
  /// return [Page] with detected polygon.
  static Future<Page> detectDocument(Page page) async {
    try {
      var arguments = {'page': page.toJson()};
      var documentScanningResult =
          await _channel.invokeMethod('detectDocument', arguments);
      var json = jsonDecode(documentScanningResult);
      var pageResult = Page.fromJson(json);
      return pageResult;
    } catch (e) {
      return page;
    }
  }

  /// Returns the current license status of the Scanbot SDK.
  static Future<SdkLicenseStatus> getLicenseStatus() async {
    try {
      var licenseStatus = await _channel.invokeMethod('getLicenseStatus', {});
      var json = jsonDecode(licenseStatus);
      return SdkLicenseStatus.fromJson(json);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Renders a multi-page PDF document from DOCUMENT images of the given [pages]
  /// and stores it as a file. For each DOCUMENT image a separate PDF page is generated.
  /// All given [pages] must have the DOCUMENT images.
  ///
  /// [pages] list of pages to render
  /// [options] additional options of rendering
  /// return File [Uri] of the PDF document.
  static Future<Uri> createPdf(
      List<Page> pages, PdfRenderingOptions options) async {
    try {
      var pagesList = pages.map((page) => page.toJson()).toList();
      var arguments = {'pages': pagesList, 'options': options.toJson()};
      final uri = await _channel.invokeMethod('createPdf', arguments);
      return Uri.parse(uri);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Renders a multi-page TIFF image from DOCUMENT images of the given [pages]
  /// and stores it as a file. For each DOCUMENT image a separate TIFF page is generated.
  /// All given [pages] must have the DOCUMENT images.
  ///
  /// [pages] list of pages to render
  /// [options] additional options of rendering
  static Future<Uri> createTiff(
      List<Page> pages, TiffCreationOptions options) async {
    try {
      if (!options.binarized &&
          PredefinedCompressionTypes.binarizedOnlyCompressionTypes()
              .contains(options.compression)) {
        throw ('Incompatible compression type. The given compression type ${options.compression} is only applicable to binarized images.');
      }

      var pagesList = pages.map((page) => page.toJson()).toList();
      var arguments = {'pages': pagesList, 'options': options.toJson()};
      final uri = await _channel.invokeMethod('createTiff', arguments);
      return Uri.parse(uri);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Cleanups the storage folders of the Scanbot SDK.
  /// By calling this function all output files generated by the native Scanbot SDKs
  /// as well as by this Plugin will be deleted, including the image files of all scanned
  /// or imported Page objects, all generated PDF files, TIFF files, etc.
  static Future<void> cleanupStorage() async {
    try {
      await _channel.invokeMethod('cleanupStorage');
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Removes a page with all its files (ORIGINAL image, DOCUMENT image,
  /// previews/thumbnails, filtered images, etc) from the file storage of the Scanbot SDK.
  /// This method does not remove any exported files (PDF, TIFF, etc) which were
  /// generated based on this page object.
  static Future<void> deletePage(Page page) async {
    try {
      var arguments = {'page': page.toJson()};
      await _channel.invokeMethod('deletePage', arguments);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Creates a page from an image file URI. The given image will be used as ORIGINAL image.
  /// Use the [shouldDetectDocument] flag to specify whether the auto-document detection
  /// should be performed and the (cropped) DOCUMENT image should be created.
  /// return [Page]
  static Future<Page> createPage(
      Uri originalImageFileUri, bool shouldDetectDocument) async {
    try {
      var arguments = {
        'originalImageFileUri': originalImageFileUri.toString(),
        'shouldDetectDocument': shouldDetectDocument
      };
      final pageJson = await _channel.invokeMethod('createPage', arguments);
      var decoded = jsonDecode(pageJson);
      return Page.fromJson(decoded);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Creates a page from image bytes. The given image will be used as ORIGINAL image.
  /// Use the [shouldDetectDocument] flag to specify whether the auto-document detection
  /// should be performed and the (cropped) DOCUMENT image should be created.
  /// return [Page]
  static Future<Page> createPageWithImageBytes(
      Uint8List imageBytes, bool shouldDetectDocument) async {
    try {
      var arguments = {
        'originalImageBytes': imageBytes,
        'shouldDetectDocument': shouldDetectDocument
      };
      final pageJson =
          await _channel.invokeMethod('createPageWithImageBytes', arguments);
      var decoded = jsonDecode(pageJson);
      return Page.fromJson(decoded);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Rotates all images of the given [page] by 90 degrees value for [rotationsCount] times.
  static Future<Page> rotatePageClockwise(Page page, int rotationsCount) async {
    try {
      var arguments = {'page': page.toJson(), 'rotationsCount': rotationsCount};
      var pageJson =
          await _channel.invokeMethod('rotatePageClockwise', arguments);
      var decoded = jsonDecode(pageJson);
      var rotatedPage = Page.fromJson(decoded);
      return rotatedPage;
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Applies an image [filter] on the final DOCUMENT image of given [page].
  static Future<Page> applyImageFilter(
      Page page, ImageFilterType filter) async {
    try {
      var arguments = {
        'page': page.toJson(),
        'filter': imageFilterJsonValue(filter)
      };
      final pageJson =
          await _channel.invokeMethod('applyImageFilter', arguments);
      var decoded = jsonDecode(pageJson);
      return Page.fromJson(decoded);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Creates a preview image file of the DOCUMENT image by given [filter].
  /// Does not modify the actual DOCUMENT image of the passed [page].
  /// Use the 'applyImageFilter(page, filter)' method to finally apply an image filter on a page.
  static Future<Uri> getFilteredDocumentPreviewUri(
      Page page, ImageFilterType filter) async {
    try {
      var arguments = {
        'page': page.toJson(),
        'filter': imageFilterJsonValue(filter)
      };
      final uri = await _channel.invokeMethod(
          'getFilteredDocumentPreviewUri', arguments);
      return Uri.parse(uri);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Returns current OCR configs, like installed OCR languages, etc.
  static Future<SdkOcrConfig> getOcrConfigs() async {
    try {
      var result = await _channel.invokeMethod('getOcrConfig');
      var json = jsonDecode(result);
      return SdkOcrConfig.fromJson(json);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Performs OCR (Optical Character Recognition) on DOCUMENT images of given pages.
  static Future<OcrResult> performOcr(
      List<Page> pages, OcrOptions options) async {
    try {
      var arguments = {
        'pages': pages.map((e) => e.toJson()).toList(),
        'options': options.toJson()
      };

      var result = await _channel.invokeMethod('performOcr', arguments);
      var json = jsonDecode(result);
      return OcrResult.fromJson(json);
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Refreshes image file URIs of given pages. For more details please
  /// check out the section 'Persistence of Page Objects' in the documentation.
  static Future<List<Page>> refreshImageUris(List<Page> pages) async {
    try {
      final jsonPages = pages.map((page) => page.toJson()).toList();
      final result =
          await _channel.invokeMethod('refreshImageUris', {'pages': jsonPages});
      final json = jsonDecode(result);
      return (json['pages'] as List?)
              ?.map((p) => Page.fromJson(p as Map<String, dynamic>))
              .toList() ??
          <Page>[];
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Detect barcodes from image file uri
  /// The Barcode Scanner is able to recognize 1D- and 2D-barcodes (QR codes, etc).
  /// image [rotation]  between 0-360 degrees with 90 degree step
  /// [barcodeFormats] Format of barcode which  need to recognize
  /// [acceptedDocumentFormats] Format of document which  need to recognize
  static Future<BarcodeScanningResult> detectBarcodesOnImage(
    Uri file,
    List<BarcodeFormat> barcodeFormats, {
    List<BarcodeDocumentFormat> acceptedDocumentFormats =
        allDocumentFormatTypes,
  }) async {
    var barcodeResult = BarcodeScanningResult.error(
        "Barcode detection failed"); // default error message initialisation
    try {
      List<Uri> list = [file];
      var result = await detectBarcodesOnImages(list, barcodeFormats);
      if (result.operationResult == OperationResult.SUCCESS) {
        if (result.barcodeResults.isNotEmpty) {
          barcodeResult = result.barcodeResults.first;
        }
      } else {
        barcodeResult.message = result.message;
      }
      return barcodeResult;
    } catch (e) {
      Logger.root.severe(e);
      barcodeResult.message = e.toString();
      return barcodeResult;
    }
  }

  /// Estimate blurriness on a given page.
  /// The document image will be analyzed, if possible. If there is no document image - then the original image will be used.
  /// Returns the value between 0.0 and 1.0. Values closer to 0.0 mean that the image is sharper, values closer to 1.0 - the image is blurred.
  static Future<double> estimateBlurOnPage(Page page) async {
    try {
      var arguments = {'page': page.toJson()};
      final value =
          await _channel.invokeMethod('estimateBlurOnPage', arguments);
      return value;
    } catch (e) {
      Logger.root.severe(e);
      rethrow;
    }
  }

  /// Detect barcodes from the given images list
  static Future<MultiImageBarcodeScanningResult> detectBarcodesOnImages(
      List<Uri> imageFileUris, List<BarcodeFormat> barcodeFormats,
      {List<BarcodeDocumentFormat> acceptedDocumentFormats =
          allDocumentFormatTypes}) async {
    try {
      var detectFromMultipleImageParams =
          DetectBarcodeFromImageParams.fromMultiple(
        imageFileUris,
        barcodeFormats,
        acceptedDocumentFormats,
      );
      var barcodeScanningResult = await _channel.invokeMethod(
          'detectBarcodesOnImages', detectFromMultipleImageParams.toJson());
      var result = MultiImageBarcodeScanningResult.fromJson(
          jsonDecode(barcodeScanningResult));
      return result;
    } catch (e) {
      Logger.root.severe(e);
      return MultiImageBarcodeScanningResult.error("parsing exception");
    }
  }
}
